import time
from random import random
import hashlib

from rivr.middleware import Middleware

class BaseSessionStore(object):
    def __init__(self, request, session_key=None):
        self.modified = False
        self.session_key = session_key
        self.data = {}

        if self.session_key:
            self.get_session(request)

    def get_session(self, request):
        raise NotImplementedError

    def save(self, request):
        raise NotImplementedError

    def __contains__(self, key):
        return key in self.data

    def __getitem__(self, key):
        return self.data[key]

    def __setitem__(self, key, value):
        self.modified = True
        self.data[key] = value

    def __delitem__(self, key):
        self.modified = True
        del self.data[key]

    def has_key(self, key):
        return key in self.data

    def clear(self):
        self.modified = True
        self.data = {}

    def generate_key(self):
        self.modified = True
        self.session_key = hashlib.new('sha1', str(time.time()) + str(random())).hexdigest()

class SessionMiddleware(Middleware):
    cookie_name = 'sessionid'
    cookie_secure = False
    cookie_path = '/'
    cookie_domain = None

    session_store = None

    def process_request(self, request):
        if self.session_store is None:
            raise Exception("SessionMiddleware is improperly configured."
                            "Session store is not defined")

        session_key = request.COOKIES.get(self.cookie_name, None)
        request.session = self.session_store(request, session_key)

    def process_response(self, request, response):
        if request.session.modified:
            # Save the session data and refresh the client cookie.
            request.session.save(request)

            response.set_cookie(self.cookie_name, request.session.session_key,
                path=self.cookie_path, domain=self.cookie_domain,
                secure=self.cookie_secure)

        return response



import math

from rivr.http import Response, ResponseRedirect
from rivr.views.base import View, TemplateView

try:
    from pymongo.objectid import ObjectId
    from pymongo import json_util
    import gridfs
except ImportError:
    ObjectId = lambda x: x
    gridfs = None


class GridFSResponse(Response):
    def __init__(self, grid_file):
        super(GridFSResponse, self).__init__(content_type=str(grid_file.content_type))
        self.grid_file = grid_file
    
    def get_content(self):
        if not hasattr(self, '_content'):
            self._content = self.grid_file.read()
        return self._content
    
    def set_content(self, value):
        pass
    content = property(get_content, set_content)


class GridFSView(View):
    def get_gridfs(self):
        return gridfs.GridFS(self.request.mongodb_database)

    def get_file(self):
        object_id = self.kwargs.get('object_id', None)
        return self.get_gridfs().get(ObjectId(object_id))

    def get(self, request, *args, **kwargs):
        return GridFSResponse(self.get_file())

class MongoMixin(object):
    collection = None

    def get_collection_name(self):
        if "mongodb_collection" in self.kwargs:
            return self.kwargs['mongodb_collection']

        if self.collection:
            return self.collection

        raise Exception("MongoMixin requires either a definition of 'collection'"
                        "or a implementation of 'get_collection_name()'")

    def get_collection(self):
        return self.request.mongodb_database[self.get_collection_name()]

class SingleObjectMixin(MongoMixin):
    slug_field = 'slug'
    context_object_name = None

    def get_object(self):
        object_id = self.kwargs.get('object_id', None)
        slug = self.kwargs.get('slug', None)
        lookup = {}

        if object_id:
            lookup['_id'] = ObjectId(object_id)
        elif slug:
            lookup[self.get_slug_field()] = slug
        else:
            raise AttributeError("%s must be called with either an object_id"
                                 "or a slug" % self.__class__.__name__)

        return self.get_collection().find_one(lookup)

    def get_slug_field(self):
        return self.slug_field
 
    def get_context_object_name(self, obj):
        if self.context_object_name:
            return self.context_object_name
        return 'object'

    def get_context_data(self, **kwargs):
        context = kwargs
        obj = self.get_object()
        context_object_name = self.get_context_object_name(obj)
        if context_object_name:
            context[context_object_name] = obj
        context['mongodb_collection'] = self.get_collection_name()
        return context

    def get_template_names(self):
        try:
            names = super(SingleObjectMixin, self).get_template_names()
        except:
            names = '%s_detail.html' % self.get_context_object_name(None)

        return names


class DetailView(SingleObjectMixin, TemplateView):
    pass

class MultipleObjectMixin(MongoMixin):
    context_object_name = None
    paginate_by = 20
    allow_filters = True

    lookup_filters = {
        'int': int,
        'gt': lambda x: {'$gt': int(x)},
        'gte': lambda x: {'$gte': int(x)},
        'lt': lambda x: {'$lt': int(x)},
        'lte': lambda x: {'$lte': int(x)},
        'regex': lambda x: {'$regex': x},
        'iregex': lambda x: {'$regex': x, '$options': 'i'},
    }

    def resolve_page(self):
        self.page = 1

        try:
            self.page = int(self.request.GET['page'])
        except (KeyError, TypeError, ValueError):
            pass

        if 'page' in self.kwargs:
            try:
                self.page = int(self.kwargs['page'])
            except:
                pass

        if self.page < 1:
            self.page = 1

    def get_object_list(self):
        self.resolve_page()

        lookup = {}

        if self.allow_filters:
            for l in self.request.GET:
                if l == 'page':
                    continue
                elif '__' in l:
                    key, f = l.split('__')
                    if f in self.lookup_filters:
                        lookup[key] = self.lookup_filters[f](self.request.GET[l])
                else:
                    lookup[l] = self.request.GET[l]

        query = self.get_collection().find(lookup)
        paged_query = query.skip(self.paginate_by * (self.page - 1)).limit(self.paginate_by)
        self.page_count = int(math.ceil(float(query.count()) / float(self.paginate_by)))
        return paged_query

    def get_context_object_name(self, object_list):
        if self.context_object_name:
            return self.context_object_name
        return 'object'

    def get_context_data(self, **kwargs):
        context = kwargs
        object_list = self.get_object_list()
        context_object_name = self.get_context_object_name(object_list)
        if context_object_name:
            context['%s_list' % context_object_name] = object_list

        context['paginate_by'] = self.paginate_by
        context['page'] = self.page
        context['page_count'] = self.page_count

        context['mongodb_collection'] = self.get_collection_name()

        return context

    def get_template_names(self):
        try:
            names = super(MultipleObjectMixin, self).get_template_names()
        except:
            names = '%s_list.html' % self.get_context_object_name(None)

        return names


class ListView(MultipleObjectMixin, TemplateView):
    pass

class DeleteView(DetailView):
    post_delete_redirect = '/'

    def post(self, request, *args, **kwargs):
        self.get_collection().remove(self.get_object())
        return ResponseRedirect(self.post_delete_redirect)

    def get_template_names(self):
        if self.template_name:
            return self.template_name
        return '%s_confirm_delete.html' % self.get_context_object_name(None)




from rivr.middleware.base import Middleware
from rivr.http import Response, ResponseNotFound, Http404

class MiddlewareController(Middleware):
    def __init__(self, handler, *middleware):
        super(MiddlewareController, self).__init__(handler)
        self.request_middleware = []
        self.response_middleware = []
        self.exception_middleware = []

        for mw_instance in middleware:
            self.append(mw_instance)

    def append(self, middleware):
        if hasattr(middleware, 'process_request'):
            self.request_middleware.append(middleware.process_request)
        if hasattr(middleware, 'process_response'):
            self.response_middleware.insert(0, middleware.process_response)
        if hasattr(middleware, 'process_exception'):
            self.exception_middleware.insert(0, middleware.process_exception)

    def process_request(self, request):
        for request_mw in self.request_middleware:
            response = request_mw(request)
            if response:
                return response

    def process_response(self, request, response):
        for response_mw in self.response_middleware:
            response = response_mw(request, response)

        return response

    def process_exception(self, request, exception):
        for exception_mw in self.exception_middleware:
            response = exception_mw(request, e)
            if response:
                return response

class ErrorWrapper(object):
    def __init__(self, app, custom_404=None, custom_500=None):
        self.app = app
        self.error_404 = custom_404 or self.default_error_404
        self.error_500 = custom_500 or self.default_error_500

    def __call__(self, request, *args, **kwargs):
        try:
            response = self.app(request, *args, **kwargs)
        except Http404, e:
            response = self.error_404(request, e)
        except Exception, e:
            response = self.error_500(request, e)

        if not response:
            response = self.error_404(request, Http404())

        return response

    def default_error_404(self, request, e):
        return ResponseNotFound("404: %s" % e)

    def default_error_500(self, request, e):
        return Response("A 500 has occured", status=500)
